//--------------------------------------------------------------------------
//
//	  Demo program to show the FT2XX drivers controlling
//	  the 8 bit port I/O. Written by Alex Drakard 2005.
//
//	  copyright stuff.... feel free to use it as you want.
//--------------------------------------------------------------------------

#include <windows.h>
#include <userint.h>
#include <formatio.h>
#include <ansi_c.h>
//#include "FTDIPanel.h"

#include "FTD2XX.h"
#include "FTDI Comms.h"

int port_val = 0;
int device_number = -1;

FT_HANDLE FTDIHandle = 0;

//--------------------------------------------------------------------------
int GetDevices(void)
{
	char *BufPtrs[MAX_DEVICES + 1];		// pointer to array of pointers 
	char Buffer[MAX_DEVICES][64];		// buffer for description of devices 
	HRESULT status;
	long num_devs;
	long i;
	char mess[1024];
	
	// initialize the array of pointers 
	for (i=0; i<MAX_DEVICES; i++) BufPtrs[i] = Buffer[i];
	BufPtrs[MAX_DEVICES] = NULL;      // last entry should be NULL 

	status = FT_ListDevices(BufPtrs, &num_devs, FT_LIST_ALL|FT_OPEN_BY_DESCRIPTION); 

	for (i=0; i<num_devs; i++)
		{
		Fmt(mess, "%s<%s", Buffer[i]);
		if (FindPattern(mess, 0,-1,DEVICE_NAME,0,0) == 0) device_number = i;
		}
	return device_number;
}

//--------------------------------------------------------------------------
int OpenDevice(int device_num)
{
	HRESULT res;
	
	if (DeviceOpen()) CloseDevice();	// close currently open device
	
	res = FT_Open(device_num, &FTDIHandle);
	if (res) MessagePopup("Error opening USB Device", "Could not open device 0");
	return res;
}

//--------------------------------------------------------------------------
int CloseDevice(void)
{
	HRESULT res;
	
	if (!DeviceOpen()) return -1; 
	res = FT_Close(FTDIHandle);
	FTDIHandle = 0;
	return res;
}

//--------------------------------------------------------------------------
int DeviceOpen(void)
{
	int res;
	
	res = 1;
	if (FTDIHandle == 0) res = 0;
	return res;
}
//--------------------------------------------------------------------------
int GetDeviceInfo(void)
{
	HRESULT res;
	FT_DEVICE  device;
	LPDWORD lpdwID;
	PCHAR SerialNumber;
	PCHAR Description;
	LPVOID Dummy;
	
	if (!DeviceOpen) return -1;
	
	lpdwID = malloc(256);
	SerialNumber = malloc(256);
	Description = malloc(256);
	
	res = FT_GetDeviceInfo(FTDIHandle, &device, lpdwID, SerialNumber, Description, &Dummy);
	return res;
}

//--------------------------------------------------------------------------
/*int IO_Setup(void)
{
	HRESULT status;
	DWORD dwIoControlCode = 255;
	LPVOID lpInBuf;
	DWORD nInBufSize = 1;
	LPVOID lpOutBuf;
	DWORD nOutBufSize = 1;
	LPDWORD lpBytesReturned = 0;
	LPOVERLAPPED lpOverlapped = 0;
	
	if (!DeviceOpen) return -1;
	status = FT_IoCtl(FTDIHandle, dwIoControlCode, &lpInBuf, nInBufSize, &lpOutBuf, 
								nOutBufSize ,lpBytesReturned ,lpOverlapped);

	return 0;
}
*/
//--------------------------------------------------------------------------
int SetBitMode(UCHAR mask, UCHAR endis)
{
	HRESULT status; 

	if (!DeviceOpen) return -1;
	status = FT_SetBitMode(FTDIHandle, mask, endis);
	return status;
} 


//--------------------------------------------------------------------------
int GetBitMode(void)
{
	UCHAR BitMode; 
	HRESULT status; 


	if (!DeviceOpen) return -1;
	status = FT_GetBitMode(FTDIHandle, &BitMode); 
	return BitMode;
} 

//--------------------------------------------------------------------------
int SetBit(int bit_num, int val)
{
	UCHAR mask;
	UCHAR enable;
	UCHAR mask_val;
	
	if (!DeviceOpen) return -1;
	if ((bit_num<0) || (bit_num>7)) return -1;
	if ((val<0) || (val>1)) return -1;
	
	mask_val = 255 - port_val;
	mask = pow(2, bit_num);
	
	if (val == 0)	//set the bit
		{
		mask = 255 - mask;
		mask_val = mask_val & mask;
		}
	else			//clear the bit
		{
		mask_val = mask_val | mask;
		}
	enable = 1;
	
	SetBitMode(mask_val, enable);
	return 0;
}

//--------------------------------------------------------------------------
int GetBit(int bit_num)
{
	int val;
	int mask;
	
	if (!DeviceOpen) return -1;
	if ((bit_num<0) || (bit_num>7)) return -1;
	mask = pow(2, bit_num);
	port_val = GetBitMode();
	val = port_val & mask;
	if (val > 0) val = 1;
	
	return val;
}
